/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.about.AboutJazzTeamServer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojo.date");
dojo.require("dojo.date.locale");
dojo.require("dojo.date.stamp");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("jazz.ui.Dialog");

dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
dojo.require("com.ibm.team.repository.web.client.internal.licensing");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.ui.internal.Util");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.about", "AboutJazzTeamServerMessages");

(function() {

var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var InitializationData = com.ibm.team.repository.web.client.init.InitializationData;
var LicenseClient = com.ibm.team.repository.web.client.internal.licensing;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var Util = com.ibm.team.repository.web.ui.internal.Util;
	
dojo.declare("com.ibm.team.repository.web.ui.internal.about.AboutJazzTeamServer", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/about/templates/AboutJazzTeamServer.html"),
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.about", "AboutJazzTeamServerMessages");
		this.forUpdatesAndInfoMsg = dojo.string.substitute(this.messages.forUpdatesAndInfo, ["<a href='http://jazz.net' target='_blank'>http://jazz.net</a>"])
	},
	
	postCreate: function() {
		if (!this.dialogTitle) this.dialogTitle = this.messages.aboutJazzTeamServer;
		this.rationalLogo.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/rtl_logo_sm.gif");
		this.rationalLogo.style.marginTop = "32px";
		this.jazzLogo.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/about/images/jazz-mark.png");
		this.jazzLogo.style.marginTop = "18px";
		this.eclipseLogo.src = dojo.moduleUrl("com.ibm.team.repository.web","ui/internal/about/images/BuiltOnEclipse_Logo.png");
		this.eclipseLogo.style.marginTop = "15px";
		this.javaLogo.src = dojo.moduleUrl("com.ibm.team.repository.web","ui/internal/about/images/sun-java-comp-logo.png");
		this.licenseKeyManagementLink.href = net.jazz.ajax._contextRoot + "/admin#action=com.ibm.team.repository.admin.manageLicenses";
		if (!Util.userIsAdmin()) {
			this.licenseKeyManagementDiv.style.display = "none";
		}
		//Keep track of the # of calls going out so we know when they are all completed.
		this.callsToExecute = 3;
		
		var srh = new ServiceResponseHandler(this, "cbGetLicenseInfo", "displayErrorDialog");
		LicenseClient.getLicenseInfo(srh);
		
		var userId = getAuthenticatedContributor().userId;
		var srh2 = new ServiceResponseHandler(this, "cbGetAssignedLicenses", "displayErrorDialog");
		LicenseClient.getAssignedLicenses(srh2, {userId: userId});
		
		var shr3 = new ServiceResponseHandler(this,"cbGetAllProductInfo","displayErrorDialog");
		ServerStatusClient.getAllProductInfo(shr3);
	},
	
	cbGetAllProductInfo: function(prodInfo) {
		this.callsToExecute--;
		this.prodInfo = prodInfo;
		this.displayAboutDialog();
	},
	
	cbGetLicenseInfo: function(licenseInfo) {
		this.callsToExecute--;
		this.licenseInfo = licenseInfo;
		this.displayAboutDialog();
	},
	
	cbGetAssignedLicenses: function(calAssignments) {
		this.callsToExecute--;
		this.assignmentIdMap = {};
		var self = this;
		dojo.forEach(calAssignments || [], function(cal) {
			self.assignmentIdMap[cal.id] = true;
		});
		this.displayAboutDialog();
	},
	
	sortProducts: function () {	
		return this.prodInfo.sort(function(a, b){
			var aName = a.name.toLowerCase();
			var bName = b.name.toLowerCase();
			return aName > bName ? 1 : -1;
		});
	},
	
	buildProductTable: function() {
		var tbody = this.productTbody;
		function col(row, node){
		    if (node === null || node === undefined) {
		        node = "-";
		    }
		    var td = document.createElement("td");
		    row.appendChild(td);
		     if (dojo.isString(node)) {
		        node = document.createTextNode(node);
		    }
		    td.appendChild(node);
		}
		this.prodInfo = this.sortProducts();
		dojo.forEach(this.prodInfo, function(product) {
			var row = document.createElement("tr");
			col(row,product.name);
			col(row,product.version);
			col(row,product.buildId);
			tbody.appendChild(row);			
		});
			
	},
	
	displayAboutDialog: function () {
		// don't execute until we have data from all remote methods
		if(this.callsToExecute > 0) {
			return;
		}

		if(this.prodInfo) {
			this.buildProductTable();
		} else {
			this.prodSection.style.display = "none";
		}
		
		var info = this.licenseInfo;		
		var serverInfo = InitializationData.get("com.ibm.team.repository.service.serverInfoInitializer");
		
		if (info.serverLicense) {
			if (info.serverLicense.productName) {
				this.serverInfo.innerHTML = info.serverLicense.productName;
			} else {
				this.serverInfo.innerHTML = "&nbsp;";
			}
			if (info.serverLicense.editionName) {
				this.editionInfo.innerHTML = info.serverLicense.editionName;
			} else {
				this.editionInfo.innerHTML = "&nbsp;";
			}
			if (info.serverLicense.description) {
				this.descriptionInfo.innerHTML = info.serverLicense.description;
			} else {
				this.descriptionInfo.innerHTML = "&nbsp;";
			}
			if (info.serverLicense.disabledReason) {
				this.licenseKeyInfo.innerHTML = dojo.string.substitute(this.messages.disabled, [info.serverLicense.disabledReason]);
				this.licenseKeyInfo.className = "errorColor";
				this.licenseRow.className = "errorRow";
			} else if (info.serverLicense.expirationTime) {
				var expirationDate = dojo.date.stamp.fromISOString(info.serverLicense.expirationTime);
				var now = new Date();
				var statusString = !expirationDate ? this.messages.permanent : this.messages.trial;

				if(expirationDate) {
					var expirationInfo = LicenseClient.getExpirationInfo(info.serverLicense.expirationTime);
					statusString += " - ";
					if(expirationInfo.isExpired) {
						statusString += dojo.string.substitute(this.messages.expiredDate, [expirationInfo.expirationDateRelative]); 
						this.licenseKeyInfo.className = "errorColor";
						this.licenseRow.className = "errorRow";
					} else {
						statusString += dojo.string.substitute(this.messages.expiresDate, [expirationInfo.expirationDateRelative]);
						this.licenseKeyInfo.className = "warningColor";
						this.licenseRow.className = "warningRow";
					}
				}
				this.licenseKeyInfo.innerHTML = statusString;
			} else {
				this.licenseKeyInfo.className = "";
				this.licenseRow.className = "";
				this.licenseKeyInfo.innerHTML = this.messages.permanent;
			}
		}
		if (info.types) {
			this.calSection.style.display = "block";
			this.licenseAssignedTo.appendChild(document.createTextNode(dojo.string.substitute(this.messages.licenseAssignedTo, [getAuthenticatedContributor().name])));
			var self = this;
			dojo.forEach(info.types, dojo.hitch(this, function (cal) {
				var tr = document.createElement("tr");
				var td = document.createElement("td");
				td.appendChild(document.createTextNode(cal.productName || ""));
				tr.appendChild(td);
				td = document.createElement("td");
				var editionStr = cal.editionName || ""
				if (editionStr && cal.variantName != null) {
					editionStr += " (" + cal.variantName + ")";
				}
				td.appendChild(document.createTextNode(editionStr));
				tr.appendChild(td);
				td = document.createElement("td");
				td.appendChild(document.createTextNode(cal.name || ""));
				tr.appendChild(td);
				td = document.createElement("td");
				td.className = "centered";
				var isAssignedToUser = self.assignmentIdMap[cal.id];
				td.appendChild(document.createTextNode(isAssignedToUser ? this.messages.yes : this.messages.no));
				tr.appendChild(td);
				this.installedCALBody.appendChild(tr);
			}));
		}
		this.showDialog();
	},
	
	displayErrorDialog: function(err) {
		if(this.errorIsDiplayed) {
			return;
		}
		this.errorIsDiplayed = true;
		var container = document.createElement("div");
		container.style.padding = "1em";
		container.className = "com-ibm-team-repository-web-ui-internal-about";
		var div = document.createElement("div");
		div.appendChild(document.createTextNode(this.messages.errorFetchingInformation));
		div.className = "errorMsg";
		container.appendChild(div);
		this.dialog = new jazz.ui.Dialog({
			contentNode: container,
			primaryTitle: this.messages.aboutJazzTeamServer,
			width: "600px"
		});
	},
	
	showDialog: function () {
		this.dialog = new jazz.ui.Dialog({
			contentNode: this.contentNode,
			primaryTitle: this.dialogTitle,
			width: "800px"
		});
	}
});

})();