/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.ConfigureTheming");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ConfigurePageMessages");

(function() {

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.ConfigureTheming", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/ConfigureTheming.html"),
	widgetsInTemplate: true,
	
	// widget initialization
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ConfigurePageMessages");
	},

	postCreate: function () {
		this.connect(this.uploadThemeWidget, "uploadSuccess", function (label) {
			this.manageThemesWidget.addNewTheme(label);
		});
	},
	
	reload: function() {
		this.manageThemesWidget.refreshThemingData();
	},
	
	show: function(args) {	
	}
});

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.UploadThemeWidget", [dijit._Widget, dijit._Templated], {
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/UploadThemeWidget.html"),
	
	constructor: function () {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ConfigurePageMessages");
		this._createAction = dojo.baseUrl + "../../_theming/create";
	},
	
	postCreate: function () {
		this._uploadButton.disabled = true;
		this._initializeUploadForm();
	},
	
	_initializeUploadForm: function () {
		this.connect(this._labelField, "onchange", "_handleUploadFieldChange");
		this.connect(this._labelField, "onkeypress", "_handleUploadFieldChange");
		this.connect(this._zipField, "onchange", "_handleUploadFieldChange");
		this.connect(this._uploadButton, "onclick", function (e) {
			if (this._labelField.value && this._labelField.value.match(/^\w+$/)) {
				this._uploadButton.disabled = true;
				this._uploadButton._labelUploaded = this._labelField.value;
				this._uploadForm.method = "post";
				dojo.io.iframe.send({
					url: dojo.baseUrl + "../../_theming/save/" + this._labelField.value,
					method: "post",
					form: this._uploadForm,
					handleAs: "text",
					handle: dojo.hitch(this, this.handleUploadResponse)
				});
			} else {
				alert(this.messages.themingInvalidLabel);
				this._labelField.focus();
			}
		});
	},
	
	_handleUploadFieldChange: function (event) {
		// use setTimeout in order to wait for the event to complete
		// before checking field values.
		setTimeout(dojo.hitch(this, function () {
			if (this._labelField.value != "" && this._zipField.value != "") {
				this._uploadButton.disabled = false;
			} else {
				this._uploadButton.disabled = true;
			}
		}), 0);
	},
	
	handleUploadResponse: function (response) {
		var status = {};
		if (response === "OK") {
			status.type = com.ibm.team.repository.web.ui.wizard.WizardStepHeader.STATUS_TYPES.OK;
			status.summary = this.messages.themeUploadSuccess;
			this.uploadSuccess(this._uploadButton._labelUploaded);
		} else {
			status.type = com.ibm.team.repository.web.ui.wizard.WizardStepHeader.STATUS_TYPES.ERROR;
			status.summary = dojo.string.substitute(this.messages.themeUploadError, [ response ]);
		}
		dijit.byId("headerArea").showStatus(status);
		this._uploadButton.disabled = false;
	},
	
	uploadSuccess: function (label) {}
});

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.ManageThemesWidget", [dijit._Widget, dijit._Templated], {
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/ManageThemesWidget.html"),
	
	constructor: function () {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ConfigurePageMessages");
	},
	
	postCreate: function () {
		this.refreshThemingData();
	},
	
	refreshThemingData: function () {
		var dfd = jazz.client.xhrGet({
			'url': dojo.baseUrl + "../../_theming/active"
		});
		dfd.addCallback(dojo.hitch(this, function (activeTheme) {
			this._activeTheme = activeTheme;
			this._fetchThemeList();
		}));
		dfd.addErrback(dojo.hitch(this, function () {
			this._activeTheme = null;
			this._fetchThemeList();
		}));
	},
	
	_fetchThemeList: function () {
		var dfdList = jazz.client.xhrGet({
			'url': dojo.baseUrl + "../../_theming/list",
			'handleAs': "json"
		});
		dfdList.addCallback(dojo.hitch(this, function (response) {
			this._populateThemingStyles(response);
		}));
		dfdList.addErrback(dojo.hitch(this, function (response) {
			status = {
				type: com.ibm.team.repository.web.ui.wizard.WizardStepHeader.STATUS_TYPES.ERROR,
				summary: this.messages.errorFetchingThemes
			};
			dijit.byId("headerArea").showStatus(status);
		}));
	},
	
	_populateThemingStyles: function (themes) {
		themes = themes.sort();
		dojox.data.dom.removeChildren(this._themeTable);
		var tr = document.createElement("tr");
		var nameTd = document.createElement("td");
		nameTd.appendChild(document.createTextNode("default"));
		tr.appendChild(nameTd);
		var actionTd = document.createElement("td");
		if (this._activeTheme != null) {
			var setDefaultTheme = document.createElement("a");
			setDefaultTheme.href = "#";
			setDefaultTheme.appendChild(document.createTextNode(this.messages.themeActionActivate));
			this.connect(setDefaultTheme, "onclick", dojo.hitch(this, function (event) {
				dojo.stopEvent(event);
				if (confirm(this.messages.confirmDefaultTheme)) {
					this._removeActiveTheme();
				}
			}));
			actionTd.appendChild(setDefaultTheme);
		}
		tr.appendChild(actionTd);
		this._themeTable.appendChild(tr);
		for (var i=0; i < themes.length; i++) {
			var tr = this._createThemeRow(themes[i]);
			this._themeTable.appendChild(tr);
		}
	},
	
	addNewTheme: function (label) {
		var tr = this._createThemeRow(label);
		this._themeTable.appendChild(tr);
	},
	
	_createThemeRow: function (themeName) {
		var tr = document.createElement("tr");
		var nameTd = document.createElement("td");
		nameTd.appendChild(document.createTextNode(themeName));
		tr.appendChild(nameTd);
		var actionTd = document.createElement("td");
		if (this._activeTheme != themeName) {
			var setActiveTheme = document.createElement("a");
			setActiveTheme.appendChild(document.createTextNode(this.messages.themeActionActivate));
			this.connect(setActiveTheme, "onclick", dojo.hitch(this, function(themeName, event) {
				dojo.stopEvent(event);
				if (confirm(dojo.string.substitute(this.messages.confirmActivateTheme, [themeName]))) {
					this._setActiveTheme(themeName);
				}
			}, themeName));
			setActiveTheme.href = "#";
			actionTd.appendChild(setActiveTheme);

			var previewTheme = document.createElement("a");
			previewTheme.style.marginLeft = "10px";
			previewTheme.appendChild(document.createTextNode(this.messages.themeActionPreview));
			this.connect(previewTheme, "onclick", dojo.hitch(this, function(themeName, event) {
				dojo.stopEvent(event);
				window.location.href = dojo.baseUrl + "../../admin?previewTheme=" + themeName + "#action=com.ibm.team.repository.admin.configureTheming"
			}, themeName));
			previewTheme.href = "#";
			actionTd.appendChild(previewTheme);

			var deleteTheme = document.createElement("a");
			deleteTheme.style.marginLeft = "10px";
			deleteTheme.appendChild(document.createTextNode(this.messages.themeActionDelete));
			this.connect(deleteTheme, "onclick", dojo.hitch(this, function(themeName, event) {
				dojo.stopEvent(event);
				if (confirm(dojo.string.substitute(this.messages.confirmDeleteTheme, [themeName]))) {
					this._deleteTheme(themeName);
				}
			}, themeName));
			deleteTheme.href = "#";
			actionTd.appendChild(deleteTheme);
		}
		tr._themeName = themeName;
		tr.appendChild(actionTd);
		return tr;
	},
	
	_setActiveTheme: function (themeName) {
		var dfd = jazz.client.xhrPost({
			'url': dojo.baseUrl + "../../_theming/active/" + themeName
		});
		dfd.addCallback(dojo.hitch(this, function (response) {
			window.location.reload(true);
		}));
	},
	
	_removeActiveTheme: function () {
		var dfd = jazz.client.xhrPost({
			'url': dojo.baseUrl + "../../_theming/disable"
		});
		dfd.addCallback(dojo.hitch(this, function (response) {
			window.location.reload(true);
		}));
	},
	
	_deleteTheme: function (themeName) {
		var dfd = jazz.client.xhrDelete({
			'url': dojo.baseUrl + "../../_theming/" + themeName
		});
		dfd.addCallback(dojo.hitch(this, function (themeName) {
			this._markThemeDeleted(themeName);
		}, themeName));
	},
	
	_markThemeDeleted: function (themeName) {
		for (var i=0; i < this._themeTable.rows.length; i++) {
			if (this._themeTable.rows[i]._themeName === themeName) {
				dojo.toggleClass(this._themeTable.rows[i], "deletedTheme", true);
				dojox.data.dom.removeChildren(this._themeTable.rows[i].cells[1]);
			}
		}
	}
});
})();