/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.ExpandingComponent");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyService");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ExpandingComponentMessages");
var messages;

var ServerPropertyService = com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyService;
var ConfigUtility = com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility;
var SRH = com.ibm.team.repository.web.transport.ServiceResponseHandler;

/*
 * KMB 1.0.1: This is poorly named as "Expanding Component" since I initially intended to make it generic.  
 * Due to time constraints, it became specific to handle floating license configuration 
 * properties.
 */

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.ExpandingComponent", [ConfigUtility, dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/ExpandingComponent.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ExpandingComponentMessages");
	},

	postCreate: function() {
		com.ibm.team.repository.web.ui.internal.admin.server.ExpandingComponent.superclass.postCreate.apply(this);
		this._initializeContent();
		dojo.connect(this, "_submitConfigCallback", this, function() {
			//this.setConnectionStatus(ConfigUtility.STATUS.SAVE_SUCCESS);
			this._initialLoadedState = ConfigUtility.STATUS.CONNECTED_OK;
		}); 
		dojo.connect(this, "_submitConfigError", this, function(){
			this.setConnectionStatus(ConfigUtility.STATUS.SAVE_ERR);
		}); 
		
		//this.connect(this._componentBody,"onkeyup","handleKeyUp");
		
		this.status = ConfigUtility.STATUS.LOADING;
	},
	
	_initializeContent: function () {
		this._services = new Array();
		var serviceConfigProps = {
			serviceInfo: {
				implementationClassName: this.implementationClassName
			},
			enableEdits: true,
			displayIn: this._componentBody,
			descriptionMode: "table", 
			initialMode: "edit",
			forceSimple: true, 
			serviceHeaderClass: ["noDisplay"], 
			hiddenPropertyList: [
				"licenseService.floatingLicenseRefreshRate"
			]
		}
		this._config = new ServerPropertyService(serviceConfigProps);
		this._services.push(this._config);
		dojo.connect(this._config, "_markChanged", this, function() {
			this.setConnectionStatusOnTimeout(ConfigUtility.STATUS.PENDING_OK);
		});	
		dojo.connect(this._config, "_checkUnchanged", this, function () {
			if (this._config._changed) {
				this.setConnectionStatusOnTimeout(ConfigUtility.STATUS.PENDING_OK);
			} else {
				this.setConnectionStatus(this._initialLoadedState);
			}
		});
		dojo.connect(this._config, "_markInvalid", this, "_handleInvalidInput");				
		dojo.connect(this._config, "_show", this, "_initializePropertiesAfterLoaded");
	},
	
	// SUPER ConfigUtility METHOD	
	_getChanges: function () {
		var changes = new Array();
		var properties = new Array();
		if (this._config._changed) {
			var changeObj = this._config._getChanges();
			changes = changeObj["changes"];
			properties = changeObj["properties"];
		}
		return {
			"changes": changes,
			"properties": properties
		};
	},
		
	_initializePropertiesAfterLoaded: function() {
		this.uriProperty = this._config.getServerPropertyElement("licenseService.floatingLicenseServerURI");
		dojox.data.dom.removeChildren(this.uriProperty._propertyName);
		this.uriProperty._propertyName.appendChild(document.createTextNode(this.messages.uriLabel));
		
		this.testConnectionButton = document.createElement("button");
		var label = document.createTextNode(this.messages.testConnection);
		this.testConnectionButton.appendChild(label);
		this.connect(this.testConnectionButton, "onclick", "_testConnection");
		this.uriProperty.actionButton.appendChild(this.testConnectionButton);	
		
		this.statusIndicationArea = this.uriProperty.statusIndicationArea;
		this.loaded();
		this._testConnection();
	},
	
	loaded: function () {
		// do nothing.  provides a hook for outer container
	},
	
	handleKeyUp: function(e) {
		if(e.target && e.target.tagName === "INPUT") {
			if(this.onKeyUpTimout){
				clearTimeout(this.onKeyUpTimout);
			}
			var f = dojo.hitch(this, this.handleKeyUpValidate);
			var timeout = 750;
			if (this.uriProperty && this.uriProperty._invalid) {
				timeout = 200;
			}
			this.onKeyUpTimout = setTimeout(f, timeout);
		}
	},
	
	handleKeyUpValidate: function() {
		if (this.uriProperty) {
			this.uriProperty._handleValueChange();
			this.uriProperty._checkValidity();
		}
	},

	
	hasURI: function () {
		return this.uriProperty.hasOverride();
	}, 
	
	// TEST CONNECTION METHODS
	_testConnection: function() {
		clearTimeout(this.onKeyUpTimout);
		this.setConnectionStatus(ConfigUtility.STATUS.TESTING);
		this._config.testRobustValidity(new SRH(this, "_testConnectionCallback", "_testConnectionError"));
	},
	
	_testConnectionCallback: function(msg) {
		var connected = null;
	
		if (msg != null) {
			connected = ConfigUtility.STATUS.CONNECTED_WARN;
			for (var i = 0; i < msg.length; i++) {
				if (msg[i].code === "ERROR") {
					connected = ConfigUtility.STATUS.CONNECTED_ERR;
				}
			}
		} else {	
			connected = this._config._changed ? ConfigUtility.STATUS.CONNECTED_OK : ConfigUtility.STATUS.CONNECTED_NO_CHANGE;
		}
		if (this._initialLoadedState == null) {
			this._initialLoadedState = connected;
		}
		this.setConnectionStatus(connected);
		if (msg != null) {
			this._showConnectionErrors(msg);
		}
	},
	
	_testConnectionError: function(errorObj) {
		this.setConnectionStatus(ConfigUtility.STATUS.CONNECTED_ERR);
		this._showConnectionErrors(this.messages.connectionError + errorObj.message);
	},
	
	_handleInvalidInput: function (msg){
		if (msg.code === "ERROR") {
			this.setConnectionStatus(ConfigUtility.STATUS.PENDING_ERR);
		} else {
			this.setConnectionStatus(ConfigUtility.STATUS.PENDING_WARN);
		}
	}, 
	
	// need this to prevent _markChanged & _checkUnchanged flickers before validation comes back with error
	setConnectionStatusOnTimeout: function(newState) {
		if(this.markChangeValidationTimeout){
			clearTimeout(this.markChangeValidationTimeout);
		}
		var f = dojo.hitch(this, function(){
			this.setConnectionStatus(newState)
		});
		var timeout = 1000;
		this.markChangeValidationTimeout = setTimeout(f, timeout);		
	},
	
	setConnectionStatus: function(newState) {
		if (this.markChangeValidationTimeout) {
			clearTimeout(this.markChangeValidationTimeout);
		}
		this._clearStatusIndication();
		if (this.uriProperty) {
			if (!this.uriProperty.hasOverride()) {
				newState = ConfigUtility.STATUS.DISABLED;
			}
		}	
		this.status = newState;
		// check required for initial load before service properties are loaded
		if (this._config != null) {
			this._allowPropertyEdit(newState.edit); 
		}		
		if (this.testConnectionButton) {
			this.testConnectionButton.disabled = (!newState.test);
		}
		this.notifyStatusChange();
		if (newState != ConfigUtility.STATUS.PENDING_ERR && newState != ConfigUtility.STATUS.PENDING_WARN) {
			this._showConnectionStatus(newState.code, this.messages[newState.messageKey]);
		}
	},
	
	notifyStatusChange: function() {
		// hook to notify widget with save button
	},
	
	configStatusIsDirty: function() {
		return this._config && this._config._changed;
	},
	
	configStatusBlocksSave: function() {
		return this._config && this._config._changed && !this.status.save; 
	},
	
	_showConnectionErrors: function(msg) {
		for (var i = 0; i < msg.length; i++) {
			var message = document.createElement("div");
			message.innerHTML = msg[i].code + ": " + msg[i].message;
			message.className = msg[i].code + "color";
			this.statusIndicationArea.appendChild(message);
		}
	},
	
	_clearStatusIndication: function () {
		dojox.data.dom.removeChildren(this.statusIndicationArea);
	},
	
	_showConnectionStatus: function(code, statusMsg) {
		if (code === "INFO") {
			code = "WARNING";
		}
		this._clearStatusIndication();
		if (statusMsg === this.messages[ConfigUtility.STATUS.DISABLED.messageKey]) {
			return;
		}
		
		var indication = document.createElement("div");
		indication.className = code+"color";
		indication.innerHTML = statusMsg;
		
		dojo.toggleClass(this.statusIndicationArea, "noDisplay", false);
		this.statusIndicationArea.appendChild(indication);
		if (code === "OK") {
			setTimeout(dojo.hitch(this, function(){
				dojo.toggleClass(this.statusIndicationArea, "noDisplay", true);
			}), 4500);
		}
	}, 
	
	getConfigWarnings: function () {
		if (this.status.code == "WARNING"){
			return [ this.messages[this.status.messageKey]];
		} else {
			return [];
		} 
	}, 
	
	getConfigErrors: function () {
		if (this.status.code == "ERROR"){
			return [ this.messages[this.status.messageKey]];
		} else {
			return [];
		} 
	}, 
	
	getConfigPendingStatus: function () {
		if (this.status === ConfigUtility.STATUS.PENDING_OK){
			return [ this.messages[this.status.messageKey]];
		} else {
			return [];
		} 
	
	},
	
	restore: function () {
		if (this.uriProperty  && this.status != ConfigUtility.STATUS.SAVE_ERR) {
			this.uriProperty.restoreSaved();
	//		this.setConnectionStatus(this._initialLoadedState);
		}
	},
	
	_allowPropertyEdit: function(allowEdits){
		// KMB FIX FOR IE FOCUS ISSUE!
//		this.uriProperty._editValueNode.disabled = !allowEdits;
	}

});

})();