/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.ServerStatus");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");
dojo.require("dojo.i18n");
dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStepHeader");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.DataTableContainer");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.util");
dojo.require("com.ibm.team.repository.web.client.internal.licensing");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.ui.HoverPopup");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ServerStatusMessages");

var WizardStepHeader = com.ibm.team.repository.web.ui.wizard.WizardStepHeader;
var STATUS_TYPES = WizardStepHeader.STATUS_TYPES;
var util = com.ibm.team.repository.web.ui.internal.admin.util;
var DataTableContainer = com.ibm.team.repository.web.ui.internal.admin.DataTableContainer;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var STATUS_ERR_IMG_SRC = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/status-error.gif");
var LIC_P = com.ibm.team.repository.web.client.internal.licensing;
var getLicenseInfo = LIC_P.getLicenseInfo;
var getExpirationInfo = LIC_P.getExpirationInfo;
var PlatformUI     = net.jazz.ajax.ui.PlatformUI;
var HoverPopup     = jazz.ui.HoverPopup;

var LARGE_NUMBER = 999999;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.ServerStatus", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/ServerStatus.html"),
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ServerStatusMessages");
		this.rssFeedTitle = this.messages.rssFeedTitle;
		this.JAZZ_ROOT = net.jazz.ajax._contextRoot;
	},

	postCreate: function() {
		com.ibm.team.repository.web.ui.internal.admin.server.ServerStatus.superclass.postCreate.apply(this);
		this.failingServiceList = [];
		this._pendingResponseCount = 0;
		this._fetchAndDisplayServerHealthInfo();
		this._fetchAndDisplayDatabaseInfo();
		this._fetchAndDisplayLicenseInfo();
		this._fetchAndDisplayProductInfo();
		
		this.header = new WizardStepHeader({
        	stepTitle: this.messages.serverStatusHeader,
			reset: dojo.hitch(this,"reload")
        });
        this.headerArea.appendChild(this.header.domNode);
	},
	
	reload: function() {
		this.header.clearMessages();
		this.failingServiceList = [];
		this._pendingResponseCount = 0;
		this._fetchAndDisplayServerHealthInfo();
		this._fetchAndDisplayDatabaseInfo();
		this._fetchAndDisplayLicenseInfo();
		this._fetchAndDisplayProductInfo();	
	},
	
	_fetchAndDisplayServerHealthInfo: function() {
		dojox.data.dom.removeChildren(this.serverStatusArea);
		this.handleLoadingData();
		var srh= new ServiceResponseHandler(this, "displayServerHealthInfo", "handleServerHealthErrorReponse");
		ServerStatusClient.getServerInfo(srh);
	},
	
	_fetchAndDisplayLicenseInfo: function() {
		dojox.data.dom.removeChildren(this.licenseSummaryArea);
		this.handleLoadingData();
		var srh= new ServiceResponseHandler(this, "displayLicenseInfo", "handleLicenseErrorReponse");
		getLicenseInfo(srh);
	},
	
	_fetchAndDisplayProductInfo: function() {
		dojox.data.dom.removeChildren(this.productInfoArea);
		this.handleLoadingData();
		var srh = new ServiceResponseHandler(this,"displayProductInfo","handleProductErrorResponse");
		ServerStatusClient.getAllProductInfo(srh);
	},
	
	_fetchAndDisplayDatabaseInfo: function() {
		dojox.data.dom.removeChildren(this.additionalDBInfoArea);
		this.handleLoadingData();
		var srh= new ServiceResponseHandler(this, "displayDatabaseStatusInfo", "handleDatabaseErrorReponse");
		ServerStatusClient.getDatabaseStatus(srh);
	},
	
	_fetchAndDisplayServiceSummaryInfo: function() {
		dojox.data.dom.removeChildren(this.serviceStatusSummary);
		var srh= new ServiceResponseHandler(this, "handleComponentResponse", "handleServiceStatusFetchError");
		ServerStatusClient.getComponents(srh)
	},
	
	_fetchAndDisplayFailingServiceInfo: function(services) {
		var srh= new ServiceResponseHandler(this, "handleServiceStatusResponse", "handleServiceStatusFetchError");
		ServerStatusClient.postStatusForServiceList(srh, services);	
	},
		
	handleLoadingData: function () {
		this._pendingResponseCount++;
		getWorkbench().showMessage(this.messages.loadingData);
	},
	
	handleDataLoaded: function() {
		this._pendingResponseCount--;
		if (this._pendingResponseCount === 0) {
			getWorkbench().hideMessageArea();
		}
	},
	
	displayServerHealthInfo: function(data) {
		dojox.data.dom.removeChildren(this.serverStatusArea);
		dojox.data.dom.removeChildren(this.serverStatusExtensionArea);
		var summaryDiv = document.createElement("div");
		
		var dbStyle = "";
		var dbStatus = this.messages.unavailable;
		if (data.databaseAvailable){
			dbStatus = this.messages.connected;
		} else {
			dbStyle = "error";
		}
		var heading = {label: this.messages.serverStatus};
		var dataBlock = [
			{label: this.messages.databaseStatus, value: dbStatus, style: dbStyle },
			{label: this.messages.serverUptime, value: util.formatDaysHoursMinsFromMilliseconds(data.uptime)}
		];
		this._appendInfoBlock(summaryDiv, heading, dataBlock);
				
		heading =  {label: this.messages.memoryUsage};
		var percentFree = Math.floor(data.freeMemory/data.totalMemory*100) + " %";
		dataBlock = [
			{label: this.messages.maxMemAlloc, value: util.formatSizeFromBytes(data.maxMemory), style: "numeric"},
			{label: this.messages.currMemAlloc, value: util.formatSizeFromBytes(data.totalMemory), style: "numeric"},
			{label: this.messages.freeMem, value: percentFree, style: "numeric"}
		];
		this._appendInfoBlock(summaryDiv, heading, dataBlock);

		this.serverStatusArea.appendChild(summaryDiv);
		
		var vmDetails = util.replaceNewLineWithBreak(data.vmInfo, true);
		var vmDetailsDiv = document.createElement("div");	
		vmDetailsDiv.innerHTML = vmDetails;
		dataBlock = [
			{label: this.messages.javaVersion, value: data.javaVersion},
			{label: this.messages.vmVendor, value: data.vmVendor},
			{label: this.messages.vmName, value: data.vmName}, 
			{label: this.messages.vmDetails, value: vmDetailsDiv}
		];
		if (data.vmErrorMessage) {
			dataBlock.unshift({label: this.messages.vmUnsupported, value: data.vmErrorMessage, style: "error"});
			this._displayErrorMessage(data.vmErrorMessage);
		}
		var vmDiv = document.createElement("div");	
		heading =  {label: this.messages.serverVM};
		this._appendInfoBlock(vmDiv, heading, dataBlock);
		this.serverStatusExtensionArea.appendChild(vmDiv);
		
		this.handleDataLoaded();
		
		this._fetchAndDisplayServiceSummaryInfo();
	},
	
	handleComponentResponse: function(components) {
		dojox.data.dom.removeChildren(this.serviceStatusSummary);
		this.failingServiceList = [];
		
		//loop thru each and call for service status for component group
		if (components && components.length > 0) {
			components.sort(function(a, b){
				return a.id.toLowerCase() > b.id.toLowerCase() ? 1 : -1;
			});
			
			var self = this;
			dojo.forEach(components, function(component) {
				var statusRequest= {
					serviceNames: []
				};
				var services = component.services;
				dojo.forEach(services, function(service){
					statusRequest.serviceNames.push(service.interfaceName);
				});
				if (statusRequest.serviceNames.length > 0){
					self._fetchAndDisplayFailingServiceInfo(statusRequest);
				}
			});
		}
	},
	
	handleServiceStatusResponse: function(statusCollection) {
		//check for failing, display if failing, add to failing count
		if(statusCollection  && statusCollection.length > 0) {
			for (var i=0; i<statusCollection.length; i++) {
				if (statusCollection[i].code != "OK") {
					this.failingServiceList.push(statusCollection[i]);
				} 
			}	
		}
		this._displayServiceStatusSummary()
	},
	
	displayDatabaseStatusInfo: function(dbStatus) {
		dojox.data.dom.removeChildren(this.additionalDBInfoArea);
		var errorClass = "";
		
		if (dbStatus.stacktrace) {
			var statusDetails = new dijit.TitlePane({title: dbStatus.message});
			var stacktrace = document.createElement("div");
			stacktrace.innerHTML = util.replaceNewLineWithPaddedBreaks(dbStatus.stacktrace, false);
			statusDetails.setContent(stacktrace);
			statusDetails.toggle();
			statusDetails.arrowNode.title = dbStatus.message;
			var statusInfo = statusDetails.domNode;
			
			this._displayErrorMessage(-1);
		} else {
			if (dbStatus.code && dbStatus.code === "ERROR") {
				errorClass = "error";
			}
			var statusInfo = dbStatus.message
		}
		var dbInfo = this._createDisplayBlock(this.messages.databaseStatus, statusInfo, errorClass);		
		
		this.additionalDBInfoArea.appendChild(dbInfo);
		this.handleDataLoaded();
	},
	
	displayProductInfo: function(prodInfo) {
		if(prodInfo) {
			var dataBlock = [];
			this.prodInfo = prodInfo;
			this.prodInfo = this.sortProducts();
			dojo.forEach(this.prodInfo, function(product) {
				dataBlock.push({
					label: product.name,
					value: product.version + " (" + product.buildId + ")"
				});		
			});
	
			var pDiv = document.createElement("div");	
			var heading = {label: this.messages.installedProducts};
			this._appendInfoBlock(pDiv, heading, dataBlock);
	
			this.productInfoArea.appendChild(pDiv);
		}	
		this.handleDataLoaded();
	},
	
	displayLicenseInfo: function(licenseInfo) {
		dojox.data.dom.removeChildren(this.licenseSummaryArea);	
		
		var licSummary = licenseInfo.serverLicense.description;
		if (licenseInfo.serverLicense.productName){
			licSummary = licenseInfo.serverLicense.productName;
		}	
		if (licenseInfo.serverLicense.editionName) {
			licSummary += " - " + licenseInfo.serverLicense.editionName;
		}
		if (licenseInfo.serverLicense.variantName){
			licSummary += " (" + licenseInfo.serverLicense.variantName + ")";
		}	 

		var dataBlock = [
			{label: this.messages.serverLicense, value: licSummary }, 
			{label: this.messages.serverLicenseStatus, value: this._getLicenseStatus(licenseInfo.serverLicense)}
		];
		
		//Only show if some CALs can expire. 
		var calStatus = this._getLicenseStatusCALS(licenseInfo);
		if(calStatus) {
			dataBlock.push({label: this.messages.clientLicenseStatus, value: calStatus });
		}
		
		if (licenseInfo.serverLicense.disabledReason && licenseInfo.serverLicense.disabledReason.length > 0) {
			dataBlock.push({label: this.messages.deactivated, value: licenseInfo.serverLicense.disabledReason, style: "error" });
		} 
		
		if (licenseInfo.floatingSetupStatus) {
			var status;
			var description;
			if (licenseInfo.floatingSetupStatus.server === true) {
				status = this.messages.configured;
				description = this.messages.configuredFloatingServer;
			} else {
				status = this.messages.notConfigured;
				var floatTabLink = "<a href='#action=com.ibm.team.repository.admin.manageLicenses&focusTab=floating'>";
				description = dojo.string.substitute(this.messages.notConfiguredFloatingServer, [ floatTabLink, "</a>" ]);
			}
						
			var licenseServerHover = this._getInfopopLink(status, description); 
			
			dataBlock.push({
				label: this.messages.floatingServer,
				value: licenseServerHover
			});
			
			var extraDetails;
			if (licenseInfo.floatingSetupStatus.client === true) {
				status = this.messages.configured;
				var licenseServerLink = "<a href=\"" + licenseInfo.floatingSetupStatus.serverURL + "\">" + licenseInfo.floatingSetupStatus.serverURL + "</a>"
				extraDetails = dojo.string.substitute(this.messages.usingLicneseServer, [ licenseServerLink ]); 
				description = dojo.string.substitute(this.messages.usingLicenseServerDescription, [ licenseServerLink ]);  
				
				if (licenseInfo.floatingSetupStatus.connectionError) {
					extraDetails = extraDetails + "<br/><span class=\"error\">" + this.messages.licenseServerError + licenseInfo.floatingSetupStatus.connectionError + "</span>";
				}
			} else {
				status = this.messages.notConfigured;
				var serverTabLink = "<a href='#action=com.ibm.team.repository.admin.manageLicenses&focusTab=server'>";
				description = dojo.string.substitute(this.messages.notConfiguredFloatingClient, [ serverTabLink, "</a>" ]);				
			}
			
			var fullStatus = document.createElement("span");
			var licenseClientHover = this._getInfopopLink(status, description);
			var hoverSpan = document.createElement("span");
			hoverSpan.appendChild(licenseClientHover);
			fullStatus.appendChild(hoverSpan);
			
			if (extraDetails) {
				var detailsSpan = document.createElement("span");
				detailsSpan.innerHTML = extraDetails;
				fullStatus.appendChild(detailsSpan);
			}
			
			
			dataBlock.push({
				label: this.messages.floatingClient,
				value: fullStatus
			});
		}
		
		var licDiv = document.createElement("div");	
		var heading = {label: this.messages.licenseStatus};
		this._appendInfoBlock(licDiv, heading, dataBlock);
		

		
		this.licenseSummaryArea.appendChild(licDiv);
		this.handleDataLoaded();
	},
	
	_getLicenseStatus: function(serverLicense) {
		if(serverLicense.expirationTime) {
			var expirationDate = dojo.date.stamp.fromISOString(serverLicense.expirationTime);
			var now = new Date();
		}
		var statusString = !expirationDate ? this.messages.permanent :
			expirationDate > now ? this.messages.trial : this.messages.expiredTrial;

		if(expirationDate) {
			var expirationInfo = getExpirationInfo(serverLicense.expirationTime);
			statusString += " - ";
			if(expirationInfo.isExpired) {
				statusString += dojo.string.substitute(this.messages.expiredDate, [expirationInfo.expirationDateRelative]); 
			} else {
				statusString += dojo.string.substitute(this.messages.expiresDate, [expirationInfo.expirationDateRelative]);
			}
		}

		return statusString;
	},
	
	_getLicenseStatusCALS: function(licenseInfo) {
		
		var minDaysRemaining = LARGE_NUMBER;
		var expInfo;
		var types = licenseInfo.types || [];
		if (licenseInfo && licenseInfo.floatingSetupStatus) {
			var floatingTypes = licenseInfo.floatingLicenseTypes || [];
        	var combined = types.concat(floatingTypes);
		} else {
			combined = types;
		}
		
		dojo.some(combined, function(type){
			if (type.expirationTime != null) {
				//Find out how many days till it expires
				var expirationInfo = getExpirationInfo(type.expirationTime);
				if (expirationInfo.isExpired) {
					minDaysRemaining = -1;
					expInfo = expirationInfo;
					return;
				} else {
					var days = expirationInfo.daysRemaining;
					if(days < minDaysRemaining ) {
						minDaysRemaining = days;
						expInfo = expirationInfo;
					}
				}
			}
		});
		
		if(minDaysRemaining == LARGE_NUMBER) {
			return false;
		}
		else {
			if(expInfo.isExpired) {
				return dojo.string.substitute(this.messages.oneOfTheCALsExpired, [expInfo.expirationDateRelative]); 
			} else {
				return dojo.string.substitute(this.messages.oneOfTheCALsExpires, [expInfo.expirationDateRelative]); 
			}
		}
	},
	
	_displayServiceStatusSummary: function() {
		dojox.data.dom.removeChildren(this.serviceStatusSummary);
		
		var status = document.createElement("div");
		status.className = "com-ibm-team-repository-admin-table";
		var statusDiv = document.createElement("div");
		statusDiv.className = "wrapper2";
		status.appendChild(statusDiv);
		var statusHeading = document.createElement("div");
		statusHeading.className = "heading headingLabel";
		statusHeading.innerHTML = this.messages.serviceErrorSummary;
		statusDiv.appendChild(statusHeading);
		
		var services = document.createElement("div");
		services.className = "failed";
			
		if (this.failingServiceList.length > 0) {
		
			// sort by name
			this.failingServiceList.sort(function(a, b){
				return a.interfaceName.toLowerCase() > b.interfaceName.toLowerCase() ? 1 : -1;
			});
			
			var self = this;
			var alternate = true;
			dojo.forEach(this.failingServiceList, function(service){
				var failedService = document.createElement("div");
				
				if (alternate) {
					failedService.className = "failed-summary alternate";
					alternate = false;
				}
				else {
					failedService.className = "failed-summary";
					alternate = true;
				}
				
				var icon = document.createElement("img");
				icon.src = STATUS_ERR_IMG_SRC;
				icon.alt = self.messages.serviceErrorIconAltText;
				failedService.appendChild(icon);
				
				var statusInfo = document.createElement("span");
				failedService.appendChild(statusInfo);
				var name = document.createElement("div");
				name.innerHTML = service.interfaceName;
				name.className = "service-name";
				statusInfo.appendChild(name);
				
				var message = document.createElement("div");
				message.innerHTML = service.message;
				message.className = "service-message";
				statusInfo.appendChild(message);
				
				services.appendChild(failedService);
			});
		} else {
			var failedService = document.createElement("div");
			services.appendChild(failedService);
			failedService.className = "failed-summary";
			failedService.innerHTML = this.messages.noServiceErrors;
		}
		statusDiv.appendChild(services);
		this.serviceStatusSummary.appendChild(status);
	},
	
	_createDisplayBlock: function(heading, data, dataClass) {
		var status = document.createElement("div");
		status.className = "com-ibm-team-repository-admin-table";
		var statusDiv = document.createElement("div");
		statusDiv.className = "wrapper2";
		status.appendChild(statusDiv);
		var statusHeading = document.createElement("div");
		statusHeading.className = "heading";
		statusHeading.innerHTML = heading;
		statusDiv.appendChild(statusHeading);
		
		
		if(isNode(data)) {
			var services = data;
		} else {
			var services = document.createElement("div");
			var failedService = document.createElement("div");
			failedService.innerHTML = data;
			if (dataClass) {
				failedService.className = "failed-summary " + dataClass;
			} else {
				failedService.className = "failed-summary";
			}
			services.appendChild(failedService);
		}
		statusDiv.appendChild(services);
		
		return status;
	},
	
	_appendInfoBlock: function(node, heading, infoList){
		var infoBlock = document.createElement("div");
		infoBlock.className = "stats-block";
		
		var args = {
			heading: heading
		};
		var infoTable = new DataTableContainer(args);
		
		var self = this;
		dojo.forEach(infoList, function(item){
			var rowData = {
				0: {content: item.label, style: "rank tight double"},
				1: {content: item.value, style: item.style}
			};
			infoTable.addRow(rowData, 2);
		});
		
		infoBlock.appendChild(infoTable.domNode);
		node.appendChild(infoBlock);
	},
	
	handleServerHealthErrorReponse: function (errorObj) {
		this._handleErrorResponse(errorObj, this.serverStatusArea, this.messages.serverStatus);
	},
	
	handleDatabaseErrorReponse: function (errorObj) {
		this._handleErrorResponse(errorObj, this.additionalDBInfoArea, this.messages.databaseStatus);
	},
	
	handleProductErrorResponse: function (errorObj) {
		this._handleErrorResponse(errorObj, this.serverStatusArea, this.messages.serverStatus);
	},
	
	handleLicenseErrorReponse: function (errorObj) {
		this._handleErrorResponse(errorObj, this.licenseSummaryArea, this.messages.serverLicense);
	},
	
	handleServiceStatusFetchError: function(errorObj) {
		this._displayErrorMessage(this.messages.errorFetchingServerStatus, errorObj);
	}, 
	
	_handleErrorResponse: function(errorObj, sectionArea, sectionHeading) {
		this._displayErrorMessage(this.messages.errorFetchingServerStatus, errorObj);

		if (errorObj && errorObj.status === 404) {
			dojox.data.dom.removeChildren(this.provisionServerMsg);
			var provisionLink = "<a href='" + this.JAZZ_ROOT + "/admin?internal=true#action=com.ibm.team.repository.admin.provisionStatus'>";
			var provisionMsg = dojo.string.substitute(this.messages.provision, [ provisionLink, "</a>" ]);
			var provision = this._createDisplayBlock(this.messages.serverStatus, provisionMsg);	
			this.provisionServerMsg.appendChild(provision);
		}
		if (errorObj && errorObj.status === 503) {
			dojox.data.dom.removeChildren(sectionArea);
			var errorBlock = this._createDisplayBlock(sectionHeading, this.messages.errorFetchingServerStatus, "error");			
			sectionArea.appendChild(errorBlock);
		}	
		this.handleDataLoaded();
	},

	_displayErrorMessage: function(message, errorObj) {
		if (!this._dbMessageSet) {
			if (message === -1) {
				this._dbMessageSet = true;
				message = this.messages.expandDBStatusInfo;
			}
			if(errorObj) {
				errorObj.message = message;
			} else {
				var errorObj = {
					message: message,
					status: STATUS_TYPES.ERROR
				};
			} 
			this.header.displayServiceCallbackErrorMessage(errorObj);
		}
	},
		
	sortProducts: function () {	
		return this.prodInfo.sort(function(a, b){
			var aName = a.name.toLowerCase();
			var bName = b.name.toLowerCase();
			return aName > bName ? 1 : -1;
		});
	},
	
	getVersionInfo: function(data) {
		var version =  "";
		if (data.version) {
			version = data.version;
		} 
		if (data.buildId) {
			var build = data.buildId;
			if (data.version) {
				build = " (" + build + ") ";
			}
			version += build;
		} 
		if (!version) {
			version = this.messages.notAvailable; 
		}
		return version;
	}, 
	
	_getInfopopLink: function(label, info, className) {
		var hoverLink = document.createElement("a");
		hoverLink.href = "#";
		hoverLink.onclick = function(){
			return false;
		};
		hoverLink.innerHTML = label;
		if (className) {
			hoverLink.className = className;
		} else {
			hoverLink.className = "infopop";
		}
		var tooltip = new HoverPopup({
			around: hoverLink,
			html: info,
			openDelay: 300,
			orientation: "R"
		}, document.createElement("span"));
		
		return hoverLink;
	}


});

function getWorkbench() {
	return net.jazz.ajax.ui.PlatformUI.getWorkbench();
}

function isNode(o){
	return o && o.nodeType && o.cloneNode;
}

})();